#include <LiquidCrystal.h>

//#define TEST

// Keys:
#define NUM_KEYS     5

#define KEY_RIGHT    0
#define KEY_UP       1
#define KEY_DOWN     2
#define KEY_LEFT     3
#define KEY_SELECT   4

// RC Channels:
#define ROLL         0
#define PITCH        1
#define YAW          2
#define THROTTLE     3
#define AUX1         4

// States:
#define NUM_STATES  30
#define ERROR       -1

// Number of Configurations: (Unknown, Tri, Quad+, QuadX, Bi, Gimbal, Y6, Hex)
#define NUM_CONFIGS  8

// Speed of the serial interface. 115200 kbit/s is the best option for a USB connection.
#define SERIAL_COM_SPEED 115200

// ----------------

LiquidCrystal lcd = LiquidCrystal(8, 9, 4, 5, 6, 7); // rs, enable, d4, d5, d6, d7
  
int adc_key_val[NUM_KEYS] = { 30, 150, 360, 535, 760 };

char emptyLine[] = "                ";
char intro    [] = "                LCD Config 0.95               ";

//State messages
char msgs[NUM_STATES][17] = {
//   0123456789012345 
    "1  Config       ", // 0
    "2  P-I-D        ", // 1
    "3  Rates        ", // 2
    "4  Sensors      ", // 3
    "5  Receiver In  ", // 4
    "6  ESC/Servo Out", // 5
    
    "Roll  P   I   D ", // 6
    "Pitch P   I   D ", // 7
    "Yaw   P   I   D ", // 8

    "D/R   Expo   ALS", // 9
    "R/P   Yaw   Thro", // 10
    
    "Gyr: Rol Pit Yaw", // 11
    "Acc: Rol Pit Z  ", // 12
    "Rho   Phi  Cycle", // 13
    
    "Rec: Rol Pit Yaw", // 14
    "Rec:  Thro  Mode", // 15
    
    "Left  Rear Right", // 16 Tri
    "     Rudder     ", // 17

    "Left       Right", // 18 Quad+
    "Front       Rear", // 19

    "Left Front Right", // 20 QuadX
    "Left  Rear Right", // 21

    "Motor 1  Motor 2", // 22 Bi
    "Servo 1  Servo 2", // 23

    "Roll       Pitch", // 24 Gimbal

    "Up:   Lf Rear Ri", // 25 Y6
    "Down: Lf Rear Ri", // 26

    "Front: Lf Mid Ri", // 27 Hex
    "Rear:  Lf Mid Ri", // 28
    
    "! Reset Values !", // 29
};

char blank       [] = " ";
char leftCursor  [] = "\177";
char rightCursor [] = "\176";
char blank2      [] = "  ";
char leftCursor2 [] = "\177 ";
char rightCursor2[] = " \176";

int8_t nextState[NUM_STATES][NUM_KEYS] = {
  // Right Up  Down Left Sel
  {   29,   5,   1,  -1,  29, }, //  0 - Config
  {    6,   0,   2,   0,   6, }, //  1 - PID
  {    9,   1,   3,   0,   9, }, //  2 - Rates
  {   11,   2,   4,   0,  11, }, //  3 - Sensors
  {   14,   3,   5,   0,  14, }, //  4 - Rec In
  {   99,   4,   0,   0,  99, }, //  5 - ESc Out

  {   80,   8,   7,   1,  80, }, //  6 - PID Roll
  {   81,   6,   8,   1,  81, }, //  7 - PID Pitch
  {   82,   7,   6,   1,  82, }, //  8 - PID Yaw

  {   83,  10,  10,   2,  83, }, //  9 - DR, Expo, AutoLevel Strength
  {   84,   9,   9,   2,  84, }, // 10 - R/P Rate, Yaw Rate, Throttle PID

  {   -1,  13,  12,   3,  -1, }, // 11 - Gyro
  {   -1,  11,  13,   3,  -1, }, // 12 - Acc
  {   -1,  12,  11,   3,  -1, }, // 13 - Angle, Cycle Time

  {   -1,  15,  15,   4,  -1, }, // 14 - R, P, Y
  {   -1,  14,  14,   4,  -1, }, // 15 - Thr, Mode

  {   -1,  17,  17,   5,  -1, }, // 16 - Tri
  {   -1,  16,  16,   5,  -1, }, // 17 - 

  {   -1,  19,  19,   5,  -1, }, // 18 - Quad+
  {   -1,  18,  18,   5,  -1, }, // 19

  {   -1,  21,  21,   5,  -1, }, // 20 - QuadX
  {   -1,  20,  20,   5,  -1, }, // 21

  {   -1,  23,  23,   5,  -1, }, // 22 - Bi
  {   -1,  22,  22,   5,  -1, }, // 23

  {   -1,  24,  24,   5,  -1, }, // 24 - Gimbal

  {   -1,  26,  26,   5,  -1, }, // 25 - Y6
  {   -1,  25,  25,   5,  -1, }, // 26

  {   -1,  28,  28,   5,  -1, }, // 27 - Hex
  {   -1,  27,  27,   5,  -1, }, // 28

  {   85,  29,  29,   0,  85, }, // 29 - Reset
};

int8_t nextOutState[NUM_CONFIGS] = {
  -1, 16, 18, 20, 22, 24, 25, 27,
};
  
char configs[NUM_CONFIGS][17] = {
//   0123456789012345 
    "                ", 
    "Tri             ", 
    "Quad+           ", 
    "QuadX           ", 
    "Bi              ", 
    "Gimb.           ",
    "Y6              ",
    "Hex             ",
};

// ----------------

static int16_t   oldKey = -1;
static int16_t   state  = 0;

static int16_t   gyroData [3] = {0,0,0};
static int16_t   angle    [2];        // absolute angle inclination in Deg
static int16_t   accSmooth[3];        // projection of smoothed and normalized gravitation force vector on x/y/z axis, as measured by accelerometer       

static int8_t    config         = 0;  // Tri, Quad, Hex, etc.
static uint8_t   nunchukPresent = 0;
static uint8_t   levelModeParam = 0;  // if level mode is a activated on the radio : channel 5 value superior to 1700

static uint8_t   P8[3], I8[3], D8[3]; // 8 bits is much faster and the code is much shorter
static uint8_t   accStrength8;        // AutoLevel Strength 0-250
static uint8_t   rollPitchRate;       // ?
static uint8_t   yawRate;             // ?
static uint8_t   dynThrPID;           // ?

static int16_t   rcHysteresis[5];     // receiver input
static uint8_t   rcRate8;             // dual rate
static uint8_t   rcExpo8;
static int16_t   altitude = 0;        // from barometer, not shown

static uint16_t  cycleTime;           // this is the number in micro second to achieve a full loop, it can differ a little and is taken into account in the PID loop
static int16_t   motor[6];            // depends on configuration 
static int16_t   servo[2] = {1500,1500};

// ----------------

void setup() {
 
  lcd.begin(16, 2); 
  lcd.clear();
  
#ifdef TEST
  lcd.setCursor(12, 1);
  lcd.print("Test");
#endif
  
  for (int i = 0; i < 32; i++) {
    lcd.setCursor(0, 0);
    lcd.print(intro + i);
    if (i != 16) {
      delay(50);
    }
    else {
      delay(750);
    }
  }

  lcd.setCursor(0, 0);
  lcd.print(msgs[state]);
  lcd.setCursor(1, 1);
  lcd.print("Reading...     ");
  lcd.setCursor(1, 1);

  Serial.begin(SERIAL_COM_SPEED);
  int readBytes = serialRead();
  if (readBytes > 0) {
    char msg[] = "Read 00 Bytes  ";
    msg[5] = (char) ('0' + readBytes/10);
    msg[6] = (char) ('0' + readBytes%10);
    lcd.print(msg);

    delay(1000);
    state = 0;
    showStatusLine();
  }
  else {
    state = ERROR;
  }
}

void loop() {
  if (state != ERROR) {
    if (state > 10) {    
      if (serialRead() > 0) {
        showStatusLine();
      }
      else {
        state == ERROR;
        return;
      }
    }
  
    int key = getKeyPressed();
    if (key >=0) {
      int next = nextState[state][key];
#ifdef TEST
      if (key == KEY_SELECT && state == 5) {
        config = (config + 1) % NUM_CONFIGS;
      }
#endif      
      if (next >= 80) { // edit values
        switch (next) {
          case 99: { state = nextOutState[config]; break; } // switch state depending on config

          case 80: { enterPid(0); break; } 
          case 81: { enterPid(1); break; } 
          case 82: { enterPid(2); break; } 

          case 83: { enterRates(0); break; } 
          case 84: { enterRates(1); break; } 
          case 85: { resetValues(); state = 0; break; } 
        }
        next = state;
      }
      
      if (next >= 0) {
        state = next;
        lcd.setCursor(0, 0);
        lcd.print(msgs[state]);
        lcd.setCursor(0, 1);
        lcd.print(emptyLine);
        showStatusLine();
      }
    }
  }
}

void resetValues() {
  P8[0] = 40; I8[0] = 30; D8[0] = 13;
  P8[1] = 40; I8[1] = 30; D8[1] = 13;
  P8[2] = 80; I8[2] =  0; D8[2] =  0;
  
  rcRate8       = 90;
  rcExpo8       = 65;
  accStrength8  = 100;
  rollPitchRate = 0;
  yawRate       = 0;
  dynThrPID     = 0;
  
  serialWrite();
}

int showStatusLine() {
  lcd.setCursor(0, 1);
  switch (state) {
    default: {
      lcd.print(emptyLine);
      break;
    }
    case 0: {
      if (config >= 0 && config < NUM_CONFIGS) {
        lcd.print(configs[config]);

        char msg[5] = "NK:0";
        msg[3] = '0' + nunchukPresent;
        lcd.setCursor(7, 1);
        lcd.print(msg);

        char msg2[] = "AL:0";
        msg2[3] = '0' + levelModeParam;
        lcd.setCursor(12, 1);
        lcd.print(msg2);
      }
      else {
        lcd.print(" Unknown Config ");
      }
      break;
    }
    case 6: { // PID Roll
      format      ( 2,  P8[0], 4, 2);    
      format      ( 7,  I8[0], 5, 4);    
      formatSigned(13, -D8[0], 3, 0);    
      break;
    }
    case 7: { // PID Pitch
      format      ( 2,  P8[1], 4, 2);    
      format      ( 7,  I8[1], 5, 4);    
      formatSigned(13, -D8[1], 3, 0);    
      break;
    }
    case 8: { // PID Yaw
      format      ( 2,  P8[2], 4, 2);    
      format      ( 7,  I8[2], 5, 4);    
      formatSigned(13, -D8[2], 3, 0);    
      break;
    }
    
    case 9: { // Dual Rate, Expo, AutoLevel Strength
      format( 0, rcRate8,      4, 0);
      format( 6, rcExpo8,      4, 0);
      format(12, accStrength8, 4, 0);
      break;
    }
    case 10: { // R/P Rate, Yaw Rate, Throttle PID
      format( 0, rollPitchRate, 4, 0);
      format( 6, yawRate,       4, 0);
      format(12, dynThrPID,     4, 0);
      break;
    }
    
    case 11: { // Gyro
      format( 0, gyroData[0], 4, 0);
      format( 6, gyroData[1], 4, 0);
      format(12, gyroData[2], 4, 0);
      break;
    }
    case 12: { // Acc
      format( 0, accSmooth[0], 4, 0);
      format( 6, accSmooth[1], 4, 0);
      format(12, accSmooth[2], 4, 0);
      break;
    }
    case 13: { // Angles, Cycle
      formatSigned( 0, angle[0]/10, 4, 0);
      formatSigned( 6, angle[1]/10, 4, 0);
      format      (12, cycleTime,   4, 0);
      break;
    }

    case 14: { // Roll, Pitch, Yaw
      formatSigned( 0, scaleServo(rcHysteresis[ROLL    ]), 4, 0); 
      formatSigned( 6, scaleServo(rcHysteresis[PITCH   ]), 4, 0);
      formatSigned(12, scaleServo(rcHysteresis[YAW     ]), 4, 0);
      break;
    }
    case 15: { // Throttle, Mode
      formatSigned( 6, scaleServo(rcHysteresis[THROTTLE]), 4, 0);
      formatSigned(12, scaleServo(rcHysteresis[AUX1    ]), 4, 0);
      break;
    }

    case 16: { // Left, Rear, Right
      formatSigned( 0, scaleServo(motor[2]), 4, 0);
      formatSigned( 6, scaleServo(motor[0]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 17: { // Rudder
      formatSigned( 6, scaleServo(servo[0]), 4, 0);
      break;
    }

    case 18: { // Left, Right
      formatSigned( 0, scaleServo(motor[2]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 19: { // Front, Rear
      formatSigned( 0, scaleServo(motor[3]), 4, 0);
      formatSigned(12, scaleServo(motor[0]), 4, 0);
      break;
    }

    case 20: { // Front Left, Right
      formatSigned( 0, scaleServo(motor[3]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 21: { // Rear Left, Right
      formatSigned( 0, scaleServo(motor[2]), 4, 0);
      formatSigned(12, scaleServo(motor[0]), 4, 0);
      break;
    }

    case 22: { // Front, Rear 
      formatSigned( 0, scaleServo(motor[0]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 23: { // Servo 0, Servo 1
      formatSigned( 0, scaleServo(servo[0]), 4, 0);
      formatSigned(12, scaleServo(servo[1]), 4, 0);
      break;
    }

    case 24: { // Roll, Pitch
      formatSigned( 0, scaleServo(servo[1]), 4, 0);
      formatSigned(12, scaleServo(servo[0]), 4, 0);
      break;
    }

    case 25: { // Up: Left, Rear, Right
      formatSigned( 0, scaleServo(motor[2]), 4, 0);
      formatSigned( 6, scaleServo(motor[0]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 26: { // Down: Left, Rear, Right
      formatSigned( 0, scaleServo(motor[5]), 4, 0);
      formatSigned( 6, scaleServo(motor[3]), 4, 0);
      formatSigned(12, scaleServo(motor[4]), 4, 0);
      break;
    }

    case 27: { // Front: Left, Mid, Right
      formatSigned( 0, scaleServo(motor[3]), 4, 0);
      formatSigned( 6, scaleServo(motor[4]), 4, 0);
      formatSigned(12, scaleServo(motor[1]), 4, 0);
      break;
    }
    case 28: { // Rear: Left, Mid, Right
      formatSigned( 0, scaleServo(motor[2]), 4, 0);
      formatSigned( 6, scaleServo(motor[5]), 4, 0);
      formatSigned(12, scaleServo(motor[0]), 4, 0);
      break;
    }
    
    case 29: {
      lcd.print("\177: cancel, \176: OK");
      break;
    }
  }
}

int scaleServo(int val) { return (val - 1500) / 5; }

void enterPid(int index) {
//  while (getKey() >= 0) {
//    delay(100);
//  }
  
  int val[3], minv[3] = { 0, 0, -40 }, maxv[3] = { 200, 250, 0 };
  val[0] = P8[index]; val[1] = I8[index]; val[2] = -D8[index];
  
  int local   = 0;
  int count   = 0;
  int lastKey = -1;
  while (local >= 0 && local < 3) {
    int key = getKey();
    if (key < 0) {
      lastKey = key;
    }
    else if (key == KEY_UP || key == KEY_DOWN || key != lastKey) {
      lastKey = key;
      count   = 0;
      boolean refresh = false;
      switch (key) {
        case KEY_LEFT  : { 
          local--; 
          break; 
        }
        case KEY_RIGHT : { 
          local++; 
          if (local < 3) { 
            break;
          }
        }
        case KEY_SELECT: { 
          lcd.setCursor(0, 1);
          lcd.print(" Writing...      ");
          
          P8[index] =  val[0];
          I8[index] =  val[1];
          D8[index] = -val[2];
          
          serialWrite();
          local = -1;
          break; 
        }
        case KEY_UP    : {
          val[local] = constrain(val[local]+1, minv[local], maxv[local]);
          refresh    = true;
          break; 
        }
        case KEY_DOWN  : { 
          val[local] = constrain(val[local]-1, minv[local], maxv[local]);
          refresh    = true;
          break; 
        }
      }
      if (refresh) {
        switch (local) {
          case 0: { format      ( 2, val[0], 4, 2); break; }
          case 1: { format      ( 7, val[1], 5, 4); break; }    
          case 2: { formatSigned(13, val[2], 3, 0); break; } 
        }
      }
    }
    
    if ((count % 20) == 0) {
      if ((count / 20) % 2 == 0) {    
        switch (local) {
          case 0: {  
            lcd.setCursor( 1, 1); lcd.print(rightCursor);
            lcd.setCursor( 6, 1); lcd.print(blank);
            break;
          }
          case 1: {
            lcd.setCursor( 1, 1); lcd.print(blank);
            lcd.setCursor( 6, 1); lcd.print(rightCursor);
            lcd.setCursor(12, 1); lcd.print(blank);
            break;
          }
          case 2: {  
            lcd.setCursor( 6, 1); lcd.print(blank);
            lcd.setCursor(12, 1); lcd.print(rightCursor);
            break;
          }
        }
      }
      else {
        lcd.setCursor( 1, 1); lcd.print(blank);
        lcd.setCursor( 6, 1); lcd.print(blank);
        lcd.setCursor(12, 1); lcd.print(blank);
      }        
    }

    count++;
    delay(50);
  }
}

void enterRates(int index) {
  int8_t val[3], maxv[3];

  maxv[0] = 100; maxv[1] = 100; maxv[2] = 100;
  
  if (index == 0) {
    val [0] = rcRate8;
    val [1] = rcExpo8;
    val [2] = accStrength8;
    maxv[2] = 250;
  }
  else {
    val [0] = rollPitchRate;
    val [1] = yawRate;
    val [2] = dynThrPID;
  }
  
  int local   = 0;
  int count   = 0;
  int lastKey = -1;
  while (local >= 0 && local < 3) {
    int key = getKey();
    if (key < 0) {
      lastKey = key;
    }
    else if (key == KEY_UP || key == KEY_DOWN || key != lastKey) {
      lastKey = key;
      count   = 0; 
      switch (key) {
        case KEY_LEFT  : { 
          local--; 
          break; 
        }
        case KEY_RIGHT : { 
          local++; 
          if (local < 3) { 
            break;
          }
        }
        case KEY_SELECT: { 
          lcd.setCursor(0, 1);
          lcd.print(" Writing...      ");
          
          if (index == 0) {
            rcRate8      = val[0];
            rcExpo8      = val[1];
            accStrength8 = val[2];
          }
          else {
            rollPitchRate = val[0];
            yawRate       = val[1];
            dynThrPID     = val[2];
          }
          
          serialWrite();
          local = -1;
          break; 
        }
        case KEY_UP    : {
          val[local] = constrain(val[local]+1, 0, maxv[local]);
          format(local * 6, val[local], 4, 0);
          break; 
        }
        case KEY_DOWN  : { 
          val[local] = constrain(val[local]-1, 0, maxv[local]);
          format(local * 6, val[local], 4, 0);
          break; 
        }
      }
    }
    
    if ((count % 20) == 0) {
      if ((count / 20) % 2 == 0) {    
        switch (local) {
          case 0: {  
            lcd.setCursor( 4, 1); lcd.print(leftCursor2); 
            lcd.setCursor(10, 1); lcd.print(blank2); 
            break;
          }
          case 1: {  
            lcd.setCursor( 4, 1); lcd.print(rightCursor2);
            lcd.setCursor(10, 1); lcd.print(leftCursor2); 
            break;
          }
          case 2: {  
            lcd.setCursor( 4, 1); lcd.print(blank2);
            lcd.setCursor(10, 1); lcd.print(rightCursor2); 
            break;
          }
        }
      }
      else {
        lcd.setCursor( 4, 1); lcd.print(blank2);
        lcd.setCursor(10, 1); lcd.print(blank2);
      }        
    }
    count++;
    delay(50);
  }
}

int getKeyPressed() {
  int key = getKey();
  waitForKeyUp(key);
  return key;
} 

void waitForKeyUp(int key) {
  if (key >= 0) {
    while (rawGetKey() == key) {
      delay(50);
    }
  }
}

int getKey() {
  int key = rawGetKey();
  if (key != oldKey) {
    delay(50);		// wait for debounce time
    key = rawGetKey();
    if (key != oldKey) {			
      oldKey = key;
    }
  }
  return key;
}

// Convert ADC value to key number
int rawGetKey() {
  unsigned int input = analogRead(0);   // read the value from the sensor  
  int result = -1;
  for (int k = 0; result < 0 && k < NUM_KEYS; k++) {
    if (input < adc_key_val[k]) {
      result = k;
    }
  }
  return result;
}

void format(int col, int val, int len, int prec) {
  char temp[len+1];
  temp[len] = '\0';
  
  val = abs(val);
  for (int i = len-1; i >= 0; i--) {
    if (len-i == prec) {
      temp[i] = '.';
    }
    else {
      int res = val % 10;
      val    /= 10;
      temp[i] = (char) ('0' + res);
    }
  }
  lcd.setCursor(col, 1);
  lcd.print(temp);
}

void formatSigned(int col, int val, int len, int prec) {
  char temp[len+1];
  temp[len] = '\0';
  if (val >= 0) {
    if (val > 0) {
      temp[0] = '+';
    }
    else {
      temp[0] = ' ';
    }
    for (int i = len-1; i >= 1; i--) {
      if (len-i == prec) {
        temp[i] = '.';
      }
      else {
        int res = val % 10;
        val    /= 10;
        temp[i] = (char) ('0' + res);
      }
    }
  }
  else {
    temp[0] = '-';
    val = -val;
    for (int i = len-1; i >= 1; i--) {
      if (len-i == prec) {
        temp[i] = '.';
      }
      else {
        int res = val % 10;
        val    /= 10;
        temp[i] = (char) ('0' + res);
      }
    }
  }
  lcd.setCursor(col, 1);
  lcd.print(temp);
}

/* ----- */

static uint8_t ptr;
static uint8_t buffer[65];

void    serialize16(int16_t a) {buffer[ptr++] = a; buffer[ptr++] = a>>8&0xff;}
void    serialize8 (int8_t  a) {buffer[ptr++] = a;}

int16_t deserialize16() {uint8_t low = buffer[ptr++]; uint8_t hi = buffer[ptr++]; return hi * 256 + low;}
int8_t  deserialize8()  {return buffer[ptr++];}

#ifndef TEST
int serialRead() {
  config = 0;

  Serial.flush();
  Serial.write('A');
  int readBytes = 0;
  int retry     = 0;
  while (!Serial.available() && retry < 300) {  // wait 30 seconds
    delay(50);
    Serial.write('A');
    delay(50);
    retry++;
  }
  
  if (retry < 300) { 
    buffer[readBytes++] = Serial.read();
    delay(2);
  
    if (buffer[0] == 'A') {
      while (readBytes < 65 && Serial.available()) {
        buffer[readBytes++] = Serial.read();
        delay(2);
      }
      if (!deserialize()) {
        readBytes = -2; // wrong terminator 'A'
      }
    }
    else {
      readBytes = -1; // wrong prefix 'A'
    }
  }
  
  if (readBytes <= 0) {
    char msg[] = "READ ERROR #0   ";
    msg[12] = (char) ('0' - readBytes);
    lcd.print(msg);
  }
  return readBytes;
}
#else
boolean myInit;

int serialRead() {
  if (!myInit) {
    myInit = true;
    ptr = 0;
    int i;
  
    serialize8('A');
    for(i=0;i<3;i++) serialize16(i);
    for(i=0;i<3;i++) serialize16(i * 300);
    serialize16(0);
    for(i=0;i<2;i++) serialize16(1000);
    for(i=0;i<6;i++) serialize16(500);
    for(i=0;i<5;i++) serialize16(i * 200);      
    serialize8(1); 
    serialize8(1);
    serialize16(4000);
    for(i=0;i<2;i++) serialize16(90);
    serialize8(1);
    for(i=0;i<3;i++) {serialize8(0);
                      serialize8(0);
                      serialize8(0);}
    serialize8(11); 
    serialize8(22);
    serialize8(33);
    serialize8(44); 
    serialize8(55);
    serialize8(66);
    serialize8('A');
      
    deserialize();
  }
  else {
    ptr = 65;
  }
  return ptr;
}
#endif

boolean deserialize() {
  ptr = 1;
  int i;
  bool succ = (true || buffer[64] == 'A');
  if (succ) {
    for(i=0;i<3;i++) accSmooth   [i] = deserialize16();
    for(i=0;i<3;i++) gyroData    [i] = deserialize16();
    altitude                         = deserialize16();
    for(i=0;i<2;i++) servo       [i] = deserialize16();
    for(i=0;i<6;i++) motor       [i] = deserialize16();
    for(i=0;i<5;i++) rcHysteresis[i] = deserialize16();      
    nunchukPresent                   = deserialize8(); 
    levelModeParam                   = deserialize8();
    cycleTime                        = deserialize16();
    for(i=0;i<2;i++) angle       [i] = deserialize16();
    config                           = deserialize8();
    for(i=0;i<3;i++) {P8         [i] = deserialize8();
                      I8         [i] = deserialize8();
                      D8         [i] = deserialize8();}
    rcRate8                          = deserialize8(); 
    rcExpo8                          = deserialize8();
    accStrength8                     = deserialize8();
    rollPitchRate                    = deserialize8(); 
    yawRate                          = deserialize8();
    dynThrPID                        = deserialize8();
    
    if (config < 0 || config >= NUM_CONFIGS) {
      config = -1; // unknown config
    }
  }
  return succ;
}

void serialWrite() {
  ptr = 0;
  serialize8('C');
  for(int i=0;i<3;i++) {serialize8(P8[i]); serialize8(I8[i]); serialize8(D8[i]);}
  serialize8(rcRate8);
  serialize8(rcExpo8);
  serialize8(accStrength8);
  serialize8(rollPitchRate);
  serialize8(yawRate);
  serialize8(dynThrPID);
  
  Serial.write(buffer, ptr);
  
  char msg[] = " Wrote 00 Bytes ";
  msg[7] = (char) ('0' + ptr/10);
  msg[8] = (char) ('0' + ptr%10);
  lcd.setCursor(0, 1);
  lcd.print(msg);
  delay(1000);
  
  serialRead();
}

